﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.Chart;
using System.Windows.Threading;
using ININ.Supervisor.Chart;

namespace IGWpfTest
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        internal class DataClass
        {
            public DataClass() {}
            public DataClass(DateTime dt, double v) { DateTime = dt; Value = v; }
            public DateTime DateTime { get; set; }
            public double Value { get; set; }
        }

        internal class Foo
        {
            public double LastValue = 50.0;
            
            public string Name { get; set; }
            public Brush Fill { get; set; }
            /*
            private AutoQueue<DataClass> _Values = new AutoQueue<DataClass>(300);
            public AutoQueue<DataClass> Values
            {
                get { return _Values; }
                set { _Values = value; }
            }

            public void AddValue(DateTime dateTime, double value)
            {
                _Values.Enqueue(new DataClass(dateTime, value));
            }
             * */
            private ObservableCollection<DataClass> _Values = new ObservableCollection<DataClass>();
            public ObservableCollection<DataClass> Values
            {
                get { return _Values; }
                set { _Values = value; }
            }

            public void AddValue(DateTime dateTime, double value)
            {
                _Values.Add(new DataClass(dateTime, value));
            }
        }

        List<Foo> _FooList = new List<Foo>();

        Random _Random = new Random();
        
        DispatcherTimer _Timer = new DispatcherTimer();
        DateTime _Current, _StartTime;
        int _IntervalInMs = 250;

        public Window1()
        {
            InitializeComponent();

            _StartTime = _Current = DateTime.Now;
            
            PrepareAxis();

            _FooList.Add(new Foo() { Name = "Foo 1", Fill = GraphBrushes.GetBrush(5), LastValue = 10.0 });
            _FooList.Add(new Foo() { Name = "Foo 2", Fill = GraphBrushes.GetBrush(6), LastValue = 100.0 });
            _FooList.Add(new Foo() { Name = "Foo 3", Fill = GraphBrushes.GetBrush(7), LastValue = 50.0 });
            _FooList.Add(new Foo() { Name = "Foo 4", Fill = GraphBrushes.GetBrush(8), LastValue = 25.0 });

            foreach (Foo foo in _FooList)
            {
                LineChart.Series.Add(CreateSeries(foo));
            }

            UpdateValues();

            _Timer.Interval = new TimeSpan(0, 0, 0, 0, _IntervalInMs);
            _Timer.Tick += new EventHandler(_Timer_Tick);
            _Timer.Start();
        }

        private Series CreateSeries(Foo foo)
        {
            Series series = new Series();
            series.DataSource = foo.Values;
            series.ChartType = ChartType.ScatterLine;
            series.Label = foo.Name;
            series.SetBinding(Series.FillProperty, new Binding("Fill") { Source = foo });
            series.StrokeThickness = 1.0;
            series.DataMapping = "ValueX = DateTime; ValueY = Value";
            return series;
        }

        private void PrepareAxis()
        {
            MoveAxis(_StartTime);
            XAxis.Unit = (XAxis.Maximum - XAxis.Minimum) / 10.0;
            XAxis.Label.Format = "{0:T}";
        }

        void _Timer_Tick(object sender, EventArgs e)
        {
            LineChart.RefreshEnabled = false;

            UpdateValues();

            MoveAxis(_Current);

            LineChart.RefreshEnabled = true;
        }

        private void UpdateValues()
        {
            _Current = DateTime.Now;

            foreach (Foo foo in _FooList)
            {
                double value = foo.LastValue + (_Random.NextDouble() * 10.0) - 5.0;
                if (value < 0.0) value = 0.0;
                if (value > 100.0) value = 100.0;
                foo.LastValue = value;
                foo.AddValue(_Current, value);
            }
        }

        private void MoveAxis(DateTime newTime)
        {
            XAxis.Minimum = (newTime - new TimeSpan(0, 0, 0, 0, _IntervalInMs * 300)).ToOADate();
            XAxis.Maximum = newTime.ToOADate();
        }

    }
}
